#!/bin/sh
#set -x
#
# Downloads and prepares all components needed for eXtensible Catalog Drupal Toolkit
# demo site
#

#
# configuration section
#
DRUPAL_NAME=xc-7.x-1.1-alpha1
DISTRO_VERSION=7.x-1.0-alpha3
SOLR_VERSION=3.6.1
DRUPAL_CORE=7.22

#
# functions
#
download() {
  if [ "$HAS_WGET" = "1" ]; then
    wget -q $1
  elif [ "$HAS_CURL" = "1" ]; then
    curl -O $1
  fi
}

download_requirement_check() {
  HAS_WGET=`command -v wget >/dev/null && echo "1" || echo "0"`
  HAS_CURL=`command -v curl >/dev/null && echo "1" || echo "0"`

  if [ "$HAS_WGET" = "0" -a "$HAS_CURL" = "0" ]; then
    echo "Please install either wget or curl"
    echo ""
    echo "Or download these files manually:"
    echo "http://ftp.drupal.org/files/projects/xc_installation-$DISTRO_VERSION-core.tar.gz"
    echo "http://extensiblecatalog.org/apache-solr-3.6.1-xc-6.x-1.3.tar.gz"
    exit;
  fi
}

requirement_check() {
  HAS_JAVA=`command -v java >/dev/null && echo "1" || echo "0"`
  if [ "$HAS_JAVA" = "0" ]; then
    echo "To run Solr Java is a must have. Please install it."
    exit;
  fi
}

download_all() {
  if [ ! -e xc_installation-$DISTRO_VERSION-core.tar.gz ]; then
    download_requirement_check
    echo "Downloading distribution"
    download http://ftp.drupal.org/files/projects/xc_installation-$DISTRO_VERSION-core.tar.gz
  fi

  if [ ! -e apache-solr-$SOLR_VERSION.tgz ]; then
    download_requirement_check
    echo "Downloading Apache Solr"
    download http://extensiblecatalog.org/apache-solr-3.6.1-xc-6.x-1.3.tar.gz
  fi
}

#
# the script
#
rm -rf $DRUPAL_NAME
rm -rf apache-solr-$SOLR_VERSION-$DRUPAL_NAME

requirement_check
download_all

CWD=`pwd`
echo "Prepare Drupal"
tar xzf xc_installation-$DISTRO_VERSION-core.tar.gz
mv drupal-$DRUPAL_CORE $DRUPAL_NAME

cd $DRUPAL_NAME
if [ ! -e sites/default/settings.php ]; then
  cp sites/default/default.settings.php sites/default/settings.php
fi
chmod a+w sites/default/
chmod a+w sites/default/settings.php

# go home directory
cd $CWD

echo "The preparation was successfull"

#
# The Solr part
#
echo "The Solr part"

echo '** extracting Solr...'
tar -zxf apache-solr-$SOLR_VERSION.tgz

SOLR_DIR=apache-solr-$SOLR_VERSION

mv $SOLR_DIR $SOLR_DIR-$DRUPAL_NAME

echo -n 'In which directory to put Apache Solr? ('$HOME'/solr): '
read SOLR_PARENT_DIR

if [ "$SOLR_PARENT_DIR" = "" ]; then
  SOLR_PARENT_DIR=$HOME/solr
fi

if [ ! -e $SOLR_PARENT_DIR ]; then
  mkdir $SOLR_PARENT_DIR
fi

if [ -e $SOLR_PARENT_DIR/$SOLR_DIR-$DRUPAL_NAME ]; then
  rm -rf $SOLR_PARENT_DIR/$SOLR_DIR-$DRUPAL_NAME
fi

mv $SOLR_DIR-$DRUPAL_NAME $SOLR_PARENT_DIR

#
# Move Drupal directory under web root
#
echo -n 'What is the Apache HTTP server web directory? (/var/www): '
read WWW_DIR

if [ "$WWW_DIR" = "" ]; then
  WWW_DIR=/var/www
fi

if [ -e $WWW_DIR/$DRUPAL_NAME ]; then
  rm -rf $WWW_DIR/$DRUPAL_NAME
fi

mv $DRUPAL_NAME $WWW_DIR

#
# Starting Solr
#
echo "Start Solr"
cd $SOLR_PARENT_DIR/$SOLR_DIR-$DRUPAL_NAME/example
./solr.sh start
cd $CWD

